#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#ifdef FIBONACCI
#   include <gmp.h>
    extern void fibonacci(mpz_t out, unsigned int n);
#elif FIBONAXXI
#   include <iostream>
#   include <gmpxx.h>
    extern mpz_class fibonacci(unsigned int n);
#else
    extern unsigned long long int fibonacci(unsigned int n);
#endif

static int timeval_subtract (struct timeval *result, struct timeval *x, struct timeval *y)
{
    /* Perform the carry for the later subtraction by updating y. */
    if (x->tv_usec < y->tv_usec) {
        int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
        y->tv_usec -= 1000000 * nsec;
        y->tv_sec += nsec;
    }
    if (x->tv_usec - y->tv_usec > 1000000) {
        int nsec = (x->tv_usec - y->tv_usec) / 1000000;
        y->tv_usec += 1000000 * nsec;
        y->tv_sec -= nsec;
    }
    
    /* Compute the time remaining to wait.
     *          tv_usec is certainly positive. */
    result->tv_sec = x->tv_sec - y->tv_sec;
    result->tv_usec = x->tv_usec - y->tv_usec;
    
    /* Return 1 if result is negative. */
    return x->tv_sec < y->tv_sec;
}

int main(int argc, char **argv)
{
    unsigned int n;
    struct timeval start, end, diff;

    if (!(argc == 2 && sscanf(argv[1], "%u", &n) == 1))
    {
        fprintf(stderr, "Usage: %s n\n", argv[0]);
        return EXIT_FAILURE;
    }

    gettimeofday(&start, NULL);
//calculation
#ifdef FIBONACCI
    mpz_t res;
    mpz_init(res);
    fibonacci(res, n);
#elif FIBONAXXI
    mpz_class res = fibonacci(n);
#else
    unsigned long long int res = fibonacci(n);
#endif
    gettimeofday(&end, NULL);
    
    timeval_subtract(&diff, &end, &start);

//printing
#ifdef FIBONACCI
    gmp_printf("%Zd\n", res);
    mpz_clear(res);
#elif FIBONAXXI
    std::cout << res << std::endl;    
#else
    printf("%llu\n", res);
#endif
    printf("%lu.%06lu\n", diff.tv_sec, diff.tv_usec);

    return EXIT_SUCCESS;
}
