#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include "maxsubstruct.h"

void setSub(MaxSubResult *m, llint sum, uint start, uint end)
{
    m->sum = sum;
    m->start = start;
    m->end = end;
}

static int timeval_subtract(struct timeval *result, struct timeval *x, struct timeval *y)
{
    /* Perform the carry for the later subtraction by updating y. */
    if (x->tv_usec < y->tv_usec) {
        int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
        y->tv_usec -= 1000000 * nsec;
        y->tv_sec += nsec;
    }
    if (x->tv_usec - y->tv_usec > 1000000) {
        int nsec = (x->tv_usec - y->tv_usec) / 1000000;
        y->tv_usec += 1000000 * nsec;
        y->tv_sec -= nsec;
    }

    /* Compute the time remaining to wait.
     *          tv_usec is certainly positive. */
    result->tv_sec = x->tv_sec - y->tv_sec;
    result->tv_usec = x->tv_usec - y->tv_usec;

    /* Return 1 if result is negative. */
    return x->tv_sec < y->tv_sec;
}



int main(int argc, char **argv)
{
    int min = 0xFFFFF7FF;
    int max = 0x00000800; //these small numbers ensure, that we can use at least 1,000,000 elements
    uint MAX_ARR;
    uint seed = time(NULL);
    int okay = 1;

    switch(argc)
    {
    case 5:  okay &= sscanf(argv[4], "%d", &max); /*@fallthrough@*/
    case 4:  okay &= sscanf(argv[3], "%d", &min); /*@fallthrough@*/
    case 3:  okay &= sscanf(argv[2], "%u", &seed); /*@fallthrough@*/
    case 2:  okay &= sscanf(argv[1], "%u", &MAX_ARR); break;
    default: okay = 0;
    }

    if(okay == 0)
    {
        fprintf(stderr, "Usage: %s array_size [seed [min [max]]] \n", argv[0]);
        return EXIT_FAILURE;
    }

    struct timeval start, end, diff;

    init(seed, min, max);
    int *arr = malloc(sizeof(int) * MAX_ARR);

    for(size_t i = 0; i < MAX_ARR; i++)
        arr[i] = getRand();

    gettimeofday(&start, NULL);
    MaxSubResult res = maxSubArray(arr, MAX_ARR);
    gettimeofday(&end, NULL);

    timeval_subtract(&diff, &end, &start);
    printf("%lu [%u, %u]\n", res.sum, res.start, res.end);
    printf("%lu.%06lu\n", diff.tv_sec, diff.tv_usec);

    return EXIT_SUCCESS;
}



